/***************************** C HEADER FILE *********************************
**
**  Project:    Logic IO RTCU Gateway Protocol
**  Filename:   GWPROTLD.H 
**  
******************************************************************************
**
**  This is the interface for the TCP/IP layer for gwprot protocol.
**
*****************************************************************************/
#ifndef _GWPROTLD__H_
#define _GWPROTLD__H_


/*****************************************************************************
**
**    MODULES USED 
**
*****************************************************************************/


/*****************************************************************************
**
**    DEFINITIONS AND MACROS
**
*****************************************************************************/


/****************************************************************************
**
**    TYPEDEFS AND STRUCTURES
**
****************************************************************************/
struct sock_inst_t;


/****************************************************************************
** 
**    EXPORTED VARIABLES
**
****************************************************************************/


/****************************************************************************
**
**    EXPORTED FUNCTIONS
**
****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// Create a new socket, this function must be called before any other gwprotld
// functions.
//
// Input:
//      None.
//
// Output:
//      conn             - Pointer to socket instance.
//
// Returns:
//      0                - Success.
//      1                - Error connecting.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short gwldCreate(struct sock_inst_t ** conn);

/////////////////////////////////////////////////////////////////////////////
// Delete a socket, after this function is called no other gwprotld function
// will work.
//
// Input:
//      conn             - Pointer to socket instance.
//
// Output:
//      None.
//
// Returns:
//      0                - Success.
//      1                - Error connecting.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short gwldDestroy(struct sock_inst_t ** conn);

/////////////////////////////////////////////////////////////////////////////
// This function will attempt to connect to the gateway using the specified
// IP-address and port. The IP-address can be symbolic, like "gw.rtcu.dk"
//
// Input:
//      conn             - Pointer to socket.
//      ip               - Zero terminate ASCII string with the IP-address.
//      port             - The port where the gateway listens for clients.
//      conn             - Socket if !INVALID_SOCKET then it will be tried
//                         closed, before new connection is created.
//      server           - Zero terminated ASCII string with the path to
//                         the PEM file needed to verify the server certificate.
//      client_cert      - Zero terminated ASCII string with the path to
//                         the PEM file with the client certificate.
//      client_key       - Zero terminated ASCII string with the path to
//                         the PEM file private key of the client certificate.
//      client_pass      - Zero terminated ASCII string with the password of
//                         the private key.
//
// Output:
//      None.
//
// Returns:
//      0                - Success.
//      1                - Error connecting.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short gwldConnect(
                           struct sock_inst_t* conn,
                           const char*         ip,
                           unsigned short      port,
                           const char*         server,
                           const char*         client_cert,
                           const char*         client_key,
                           const char*         client_pass
                          );

/////////////////////////////////////////////////////////////////////////////
// This function will disconnect from the gateway.
//
// Input:
//      conn             - Pointer to socket.
//
// Output:
//      None.
//
// Returns:
//      0                - Success.
//      1                - Error disconnecting.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short gwldDisconnect(struct sock_inst_t* conn);

/////////////////////////////////////////////////////////////////////////////
// This function will disconnect the connection to the internet.
// It is assumed that the line driver will automatically reconnect.
//
// Input:
//      conn             - Pointer to socket.
//
// Output:
//      None.
//
// Returns:
//      0                - Success.
//      1                - Error hanging up internet connection.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short gwldDisconnectInternet(struct sock_inst_t* conn);

/////////////////////////////////////////////////////////////////////////////
// This function will query the line driver whether the connection to the
// internet is present.
//
// Input:
//      conn             - Pointer to socket.
//
// Output:
//      None.
//
// Returns:
//      0                - Internet is available.
//      1                - Internet is not available.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short gwldIsInternetPresent(struct sock_inst_t* conn);

/////////////////////////////////////////////////////////////////////////////
// This function will query the line driver whether the connection to the
// internet is temporarily suspended.
//
// Input:
//      conn             - Pointer to socket.
//
// Output:
//      None.
//
// Returns:
//      0                - Internet is suspended.
//      1                - Internet is not suspended.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short gwldIsInternetSuspended(struct sock_inst_t* conn);

/////////////////////////////////////////////////////////////////////////////
// This function will send data to the gateway.
//
// Input:
//      conn             - Pointer to socket.
//      data             - Pointer to buffer containing the data to send.
//      data_len         - The number of bytes to send.
//
// Output:
//      None.
//
// Returns:
//      0                - Data was sent successfully.
//      1                - Data was not sent successfully.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short gwldSend(
                        struct sock_inst_t* conn,
                        const void*         data,
                        unsigned short      data_len
                       );

/////////////////////////////////////////////////////////////////////////////
// This function will receive data from the gateway.
// This function will block until data is available.
//
// Input:
//      conn             - Pointer to socket.
//      max_data_len     - The size of the data buffer.
//
// Output:
//      data             - Pointer to buffer where the received data is stored.
//      data_len         - The number of bytes received.
//
// Returns:
//      0                - Data was received successfully.
//      1                - Data was not received successfully.
//
/////////////////////////////////////////////////////////////////////////////
unsigned short gwldReceive(
                           struct sock_inst_t* conn,
                           unsigned short      max_data_len,
                           void*               data,
                           unsigned short*     data_len
                          );


#endif  /* _GWPROTLD__H_ */
/****************************************************************************
**
**    EOF 
**
****************************************************************************/
