
// Includes
#include "compress.h"

// Defines
#define FLAG_COMPRESS         0x40

/*  -------------------------------------------------------------------------
 *  get_match -- local
 *
 *  Finds a match for the bytes at the specified offset.
 */
static byte get_match( const byte* source, word ptr, word source_size, short* hash, word* size, short* pos )
{
    word hash_value;

    hash_value = (word) (40543L * (long) ((((source [ptr]      << 4) ^
                                             source [ptr + 1]) << 4) ^
                                             source [ptr + 2]) >> 4) & 0xfff;
    *pos = hash [hash_value];
    hash [hash_value] = ptr;
    if ((word) ((*pos != -1) && ((ptr - *pos)) < 4096U)) {
        for (*size = 0;; (*size)++)
            if ((*size >= 18)
            || ((word) (ptr + *size) >= source_size)
            || (source [ptr + *size] != source [*pos + *size]))
                break;

        return (byte) (*size >= 3);
    }
    return 0;
}

/*  ---------------------------------------------------------------------[<]-
    Function: compress_block

    Synopsis: Takes up to 64Kb of uncompressed data in Source, compresses
    it using a fast LZ/RLE algorithm and places the result in Dest.  The
    compression technique is comparable to that used by Zip and such tools,
    but less agressive.  It is, however, fast enough to use in realtime.
    Returns the size of the compressed data.  To decompress the data, use
    the expand_block() function.
    ---------------------------------------------------------------------[>]-*/
word compress_block( const byte* src, byte* dst, word src_size )
{
    short Hash [4096];
    short SymbolAddress;
    word  Key;
    word  Size;
    byte  Bit = 0;
    word  Command = 0;
    word  src_index = 0;
    word  dst_size = 3;
    word  HeaderIndex = 1;

    dst [0] = FLAG_COMPRESS;
    for( Key = 0; Key < 4096; Key++ )
        Hash[Key] = -1;

    while( (src_index < src_size) && (dst_size <= src_size) ) {
        if( Bit > 15 ) {
            dst[HeaderIndex]     = (byte)((Command >> 8) & 0x00ff);
            dst[HeaderIndex + 1] = (byte)( Command       & 0x00ff);
            HeaderIndex = dst_size;
            dst_size += 2;
            Bit = 0;
        }
        for( Size = 1;; Size++ )
            if( (word) (src_index + Size) >= src_size
            || (src [src_index] != src [src_index + Size])
            || (Size >= 0x0fff))
                break;

        if( Size >= 16 ) {
            dst[dst_size++] = 0;
            dst[dst_size++] = (byte)(((word) (Size - 16) >> 8) & 0x00ff);
            dst[dst_size++] = (byte)((Size - 16) & 0x00ff);
            dst[dst_size++] = src[src_index];
            src_index += Size;
            Command = (Command << 1) + 1;
        }
        else
        if( get_match(src,src_index,src_size,Hash,&Size,&SymbolAddress) != 0 ) {
            Key = ((src_index - SymbolAddress) << 4) + (Size - 3);
            dst[dst_size++] = (byte) ((Key >> 8) & 0x00ff);
            dst[dst_size++] = (byte) (Key & 0x00ff);
            src_index += Size;
            Command = (Command << 1) + 1;
        } else {
            dst[dst_size++] = src[src_index++];
            Command = (Command << 1);
        }
        Bit++;
    }
    Command <<= (16 - Bit);
    dst[HeaderIndex]     = (byte) ((Command >> 8) & 0x00ff);
    dst[HeaderIndex + 1] = (byte) ( Command       & 0x00ff);

    if( dst_size > src_size )
        return src_size;

    return (dst_size);
}


/*  ---------------------------------------------------------------------[<]-
    Function: expand_block

    Synopsis: Expands a block of data previously compressed using the
    compress_block() function.  The compressed block is passed in src;
    the expanded result in dst.  dst must be large enough to accomodate
    the largest possible decompressed block.  Returns the size of the
    uncompressed data.
    ---------------------------------------------------------------------[>]-*/
word expand_block( const byte* src, byte* dst, word src_size )
{
    word SymbolAddress;
    word ChunkSize;
    word Counter;
    word Command = 0;
    word src_index = 1;
    word dst_size = 0;
    byte Bit = 0;

    while( src_index < src_size ) {
        if( Bit == 0 ) {
            Command  = src[src_index++] << 8;
            Command += src[src_index++];
            Bit = 16;
        }
        if(Command & 0x8000) {
            SymbolAddress =  (word)(src [src_index++] << 4);
            SymbolAddress += (word)(src [src_index] >> 4);
            if(SymbolAddress) {
                ChunkSize = (word)(src[src_index++] & 0x0f) + 3;
                SymbolAddress = dst_size - SymbolAddress;
                for( Counter = 0; Counter < ChunkSize; Counter++ )
                    dst[dst_size++] = dst[SymbolAddress++];
            } else {
                ChunkSize  = (word)(src [src_index++] << 8);
                ChunkSize += (word)(src [src_index++] + 16);
                for( Counter = 0; Counter < ChunkSize; Counter++ )
                    dst[dst_size++] = src[src_index];
                src_index++;
            }
        }
        else
            dst [dst_size++] = src [src_index++];

        Command <<= 1;
        Bit--;
    }
    return (dst_size);
}
