/***************************** C SOURCE FILE *********************************
**
**  Project:    Logic IO RTCU GPRS Gateway Protocol
**  Filename:   TEST.C 
**  Version:    1.00
**  Date:       September 12, 2007
**  
******************************************************************************
**
**  VERSION HISTORY:
**  ----------------
**  Version:    1.00
**  Date:       September 12, 2007
**  Revised by: Michael Olsen
**  Description:
**      - Original version.
**
**  Version:    2.00
**  Date:       Feburary 1, 2019
**  Revised by: Brian Weidemann
**  Description:
**      - Added support for secure connections.
**
*****************************************************************************/

#define WIN32_LEAN_AND_MEAN


/*****************************************************************************
**
** MODULES USED 
**
*****************************************************************************/
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <process.h>
#include "..\gwprot\gwprot.h"


/*****************************************************************************
**
**    DEFINITIONS AND MACROS
**
*****************************************************************************/
#define TEST_DEBUG      1

#if TEST_DEBUG
  #define debug(x)      printf x
#else
  #define debug(x)
#endif


/****************************************************************************
**
**    TYPEDEFS AND STRUCTURES
**
****************************************************************************/
#pragma pack(1)

struct vsms {
  unsigned char type;
  unsigned char length;
  unsigned char data[161];
};

#pragma pack()


/*****************************************************************************
**
** PROTOTYPES OF LOCAL FUNCTIONS 
**
*****************************************************************************/
static void ShowRC(int code);
static int  gwcSendSMS(const char* number, const char* message, unsigned char msglen);
static void thServer(void* arg);


/*****************************************************************************
**
** EXPORTED VARIABLES 
**
*****************************************************************************/


/*****************************************************************************
**
** GLOBAL VARIABLES 
**
*****************************************************************************/


/*****************************************************************************
**
** EXPORTED FUNCTIONS 
**
*****************************************************************************/

int main(int argc, char* argv[])
{
  // Var
  unsigned long  j;
  unsigned short rc;
  unsigned char  dst[80], buf[80];
  int            count = 0;

  // Validate input
  if (argc != 4) {
    printf("Wrong parameters!\n\n");
    printf("test [Node ID] [RTCU Communication Hub IP] [Destination]\n");

    // Completed
    return 1;
  }

  // Configure the secure connection to use the RCH default certificate.
  // Remember to check if the correct port is used in the gwOpen function below
  // Comment out to disable secure connection.
  gwSetTLSParameters("Logic IO RCH CA.pem", NULL, NULL, NULL);

  // Open RTCU Communication Hub connection
  rc = gwOpen(atoi(argv[1]), argv[2], 5003, "AABBCCDD");
  ShowRC(rc);

  // Wait for connection
  sprintf(dst, "@%s", argv[3]);
  printf("Waiting for connection");
  fflush(stdout);
  while (gwIsConnected() != GWRC_OK) {
    Sleep(1000);
    printf(".");
    fflush(stdout);
  }
  printf("\n");

  // Start server thread
  _beginthread(thServer, 0x20000, (void*)0);
  if (strlen(dst) < 3) {
     while (1) Sleep(10000);
  }

  // Start to send
  printf("Sending SMS to %s\n",dst);
  for (j=0;;j++) {
     sprintf(buf,"Test %d",j);
     rc = gwcSendSMS(dst,buf,0);
     printf("gwSendSMS rc=%d (msg=%s)\n", rc, buf);
     //Sleep(1000);
  }

  // Completed
  return 0;
}


/*****************************************************************************
**
** LOCAL FUNCTIONS 
**
*****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// Print gwprot.dll return code to console
//
// Input:
//      code      - gwprot.dll function return code.
//
// Output:
//      None.
//
// Returns:
//      None.
//
/////////////////////////////////////////////////////////////////////////////
static void ShowRC(int code)
{
  switch (code) {
    case GWRC_OK:       printf("GWRC_OK"); break;
    case GWRC_ERROR:    printf("GWRC_ERROR"); break;
    case GWRC_NOT_CON:  printf("GWRC_NOT_CON"); break;
    case GWRC_TIMEOUT:  printf("GWRC_TIMEOUT"); break;
    case GWRC_INV_LEN:  printf("GWRC_INV_LEN"); break;
    case GWRC_IS_OPEN:  printf("GWRC_IS_OPEN"); break;
    case GWRC_NOT_OPEN: printf("GWRC_NOT_OPEN"); break;
    default:            printf("Unknown rc=%i", code); break;
  }
  printf("\n");
}

/////////////////////////////////////////////////////////////////////////////
// Send a VSMS SMS over the internet.
//
// Input:
//      number    - ASCII string with the NodeID.
//      message   - The message to send.
//      msglen    - if > 0, indicates length of message (simple 8 bit, max 140 bytes)
//                  if == 0, send message as a 7 bit text message (max 160 chars)
//
// Output:
//      None.
//
// Returns:
//      0 if successfull
//      1 general error
//      2 error on gsm net
//
/////////////////////////////////////////////////////////////////////////////
static int gwcSendSMS(const char* number, const char* message, unsigned char msglen)
{
  // Var
  static struct vsms sms;
  unsigned long  dst;
  unsigned short rcc;
  int            rc;

  // Convert destination
  dst = atol(&number[1]);
  if (!dst) {
    debug(("gwcSendSMS. dst==0\n"));
    // Completed
    return 1;
  }

  // Build PDU message
  if (msglen) {
    memcpy(sms.data, message, msglen);
    sms.type   = 1;
    sms.length = msglen;
  // Build SMS message
  } else {
    strcpy((char*)sms.data, message); 
    sms.type   = 0;
    sms.length = strlen((char*)sms.data) + 1;
  }

  // Send message
  rc = gwTransaction(0, dst, 0x17, &sms, sizeof(sms) - sizeof(sms.data) + sms.length, &rcc, 0, 0);
  printf("rc=%d\n",rc);
  if (rc != GWRC_OK) return 2;
  if (rcc) return 1;

  // Completed
  return 0;
}  

/////////////////////////////////////////////////////////////////////////////
// Print a received VSMS SMS to console.
//
// Input:
//      sms       - VSMS RACP message.
//      src_id    - The Node ID of sender.
//
// Output:
//      None.
//
// Returns:
//      0 if successfull
//
/////////////////////////////////////////////////////////////////////////////
static unsigned short receivedVSMS(struct vsms* sms,unsigned long src_id)
{
  // Var
  char buf[22];

  sprintf(buf,"@%lu",src_id);
  debug(("SMS received. msg=%s\n",sms->data));

  // Completed
  return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Server thread
//
// Input:
//      None.
//
// Output:
//      None.
//
// Returns:
//      None.
//
/////////////////////////////////////////////////////////////////////////////
static void thServer(void* arg)
{
  // Var
  char           buffer[GW_MAXDATASIZE];
  unsigned long  src_node, TNO;
  unsigned short cmd, data_len;
  int            rc;

  // Dummy
  arg = arg;

  // Starup
  Sleep(10000);
  debug(("thServer is running\n"));

  // Main loop
  for (;;) {
    // Wait for request 
    rc = gwReceiveRequest(0, &src_node, &TNO, &cmd, buffer, &data_len);
    if (rc != GWRC_OK) {
      // Receive failed
      debug(("gwReceiveRequest failed. rc=%d\n",rc));
      Sleep(100);
    } else {
      // Parse command
      switch (cmd) {
        case 0x17:  //send vsms.
          rc = gwSendResponse(src_node, TNO, receivedVSMS((struct vsms*)buffer, src_node), 0, 0);
          if (rc) debug(("gwSendResponse failed rc=%d\n",rc));
          break;

        default:
          debug(("Unknown cmd=%d\n", cmd));
          rc = gwSendResponse(src_node, TNO, 1, 0, 0);
          if (rc) debug(("gwSendResponse failed rc=%d\n", rc));
          break;
      }
    }
  }
}


/****************************************************************************
**
**    EOF 
**
****************************************************************************/